package org.apache.bcel.generic;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import org.apache.bcel.Const;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.util.ByteSequence;
import org.apache.commons.lang3.ArrayUtils;

/* loaded from: org_apache_bcel_generic.jar:org/apache/bcel/generic/InstructionList.class */
public class InstructionList implements Iterable<InstructionHandle> {
    private InstructionHandle start;
    private InstructionHandle end;
    private int length;
    private int[] bytePositions;
    private List<InstructionListObserver> observers;

    public static InstructionHandle findHandle(InstructionHandle[] ihs, int[] pos, int count, int target) {
        int l = 0;
        int r = count - 1;
        do {
            int i = (l + r) >>> 1;
            int j = pos[i];
            if (j == target) {
                return ihs[i];
            }
            if (target < j) {
                r = i - 1;
            } else {
                l = i + 1;
            }
        } while (l <= r);
        return null;
    }

    public InstructionList() {
    }

    public InstructionList(BranchInstruction i) {
        append(i);
    }

    public InstructionList(byte[] code) {
        InstructionHandle ih;
        int count = 0;
        try {
            ByteSequence bytes = new ByteSequence(code);
            Throwable th = null;
            try {
                try {
                    InstructionHandle[] ihs = new InstructionHandle[code.length];
                    int[] pos = new int[code.length];
                    while (bytes.available() > 0) {
                        int off = bytes.getIndex();
                        pos[count] = off;
                        Instruction i = Instruction.readInstruction(bytes);
                        if (i instanceof BranchInstruction) {
                            ih = append((BranchInstruction) i);
                        } else {
                            ih = append(i);
                        }
                        ih.setPosition(off);
                        ihs[count] = ih;
                        count++;
                    }
                    if (bytes != null) {
                        if (0 != 0) {
                            try {
                                bytes.close();
                            } catch (Throwable th2) {
                                th.addSuppressed(th2);
                            }
                        } else {
                            bytes.close();
                        }
                    }
                    this.bytePositions = Arrays.copyOf(pos, count);
                    for (int i2 = 0; i2 < count; i2++) {
                        if (ihs[i2] instanceof BranchHandle) {
                            BranchInstruction bi = (BranchInstruction) ihs[i2].getInstruction();
                            int target = bi.getPosition() + bi.getIndex();
                            InstructionHandle ih2 = findHandle(ihs, pos, count, target);
                            if (ih2 == null) {
                                throw new ClassGenException("Couldn't find target for branch: " + bi);
                            }
                            bi.setTarget(ih2);
                            if (bi instanceof Select) {
                                Select s = (Select) bi;
                                int[] indices = s.getIndices();
                                for (int j = 0; j < indices.length; j++) {
                                    int target2 = bi.getPosition() + indices[j];
                                    InstructionHandle ih3 = findHandle(ihs, pos, count, target2);
                                    if (ih3 == null) {
                                        throw new ClassGenException("Couldn't find target for switch: " + bi);
                                    }
                                    s.setTarget(j, ih3);
                                }
                            } else {
                                continue;
                            }
                        }
                    }
                } finally {
                }
            } finally {
            }
        } catch (IOException e) {
            throw new ClassGenException(e.toString(), e);
        }
    }

    public InstructionList(CompoundInstruction c) {
        append(c.getInstructionList());
    }

    public InstructionList(Instruction i) {
        append(i);
    }

    public void addObserver(InstructionListObserver o) {
        if (this.observers == null) {
            this.observers = new ArrayList();
        }
        this.observers.add(o);
    }

    public BranchHandle append(BranchInstruction i) {
        BranchHandle ih = BranchHandle.getBranchHandle(i);
        append(ih);
        return ih;
    }

    public InstructionHandle append(CompoundInstruction c) {
        return append(c.getInstructionList());
    }

    public InstructionHandle append(Instruction i) {
        InstructionHandle ih = InstructionHandle.getInstructionHandle(i);
        append(ih);
        return ih;
    }

    public InstructionHandle append(Instruction i, CompoundInstruction c) {
        return append(i, c.getInstructionList());
    }

    public InstructionHandle append(Instruction i, Instruction j) {
        return append(i, new InstructionList(j));
    }

    public InstructionHandle append(Instruction i, InstructionList il) {
        InstructionHandle ih = findInstruction2(i);
        if (ih == null) {
            throw new ClassGenException("Instruction " + i + " is not contained in this list.");
        }
        return append(ih, il);
    }

    private void append(InstructionHandle ih) {
        if (isEmpty()) {
            this.end = ih;
            this.start = ih;
            ih.setNext(ih.setPrev(null));
        } else {
            this.end.setNext(ih);
            ih.setPrev(this.end);
            ih.setNext(null);
            this.end = ih;
        }
        this.length++;
    }

    public BranchHandle append(InstructionHandle ih, BranchInstruction i) {
        BranchHandle bh = BranchHandle.getBranchHandle(i);
        InstructionList il = new InstructionList();
        il.append(bh);
        append(ih, il);
        return bh;
    }

    public InstructionHandle append(InstructionHandle ih, CompoundInstruction c) {
        return append(ih, c.getInstructionList());
    }

    public InstructionHandle append(InstructionHandle ih, Instruction i) {
        return append(ih, new InstructionList(i));
    }

    public InstructionHandle append(InstructionHandle ih, InstructionList il) {
        if (il == null) {
            throw new ClassGenException("Appending null InstructionList");
        }
        if (il.isEmpty()) {
            return ih;
        }
        InstructionHandle next = ih.getNext();
        InstructionHandle ret = il.start;
        ih.setNext(il.start);
        il.start.setPrev(ih);
        il.end.setNext(next);
        if (next != null) {
            next.setPrev(il.end);
        } else {
            this.end = il.end;
        }
        this.length += il.length;
        il.clear();
        return ret;
    }

    public InstructionHandle append(InstructionList il) {
        if (il == null) {
            throw new ClassGenException("Appending null InstructionList");
        }
        if (il.isEmpty()) {
            return null;
        }
        if (isEmpty()) {
            this.start = il.start;
            this.end = il.end;
            this.length = il.length;
            il.clear();
            return this.start;
        }
        return append(this.end, il);
    }

    private void clear() {
        this.end = null;
        this.start = null;
        this.length = 0;
    }

    public boolean contains(Instruction i) {
        return findInstruction1(i) != null;
    }

    public boolean contains(InstructionHandle i) {
        if (i == null) {
            return false;
        }
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                if (ih != i) {
                    instructionHandle = ih.getNext();
                } else {
                    return true;
                }
            } else {
                return false;
            }
        }
    }

    public InstructionList copy() {
        Map<InstructionHandle, InstructionHandle> map = new HashMap<>();
        InstructionList il = new InstructionList();
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih == null) {
                break;
            }
            Instruction c = ih.getInstruction().copy();
            if (c instanceof BranchInstruction) {
                map.put(ih, il.append((BranchInstruction) c));
            } else {
                map.put(ih, il.append(c));
            }
            instructionHandle = ih.getNext();
        }
        InstructionHandle ih2 = this.start;
        InstructionHandle instructionHandle2 = il.start;
        while (true) {
            InstructionHandle ch = instructionHandle2;
            if (ih2 != null) {
                Instruction i = ih2.getInstruction();
                Instruction c2 = ch.getInstruction();
                if (i instanceof BranchInstruction) {
                    BranchInstruction bi = (BranchInstruction) i;
                    BranchInstruction bc = (BranchInstruction) c2;
                    InstructionHandle itarget = bi.getTarget();
                    bc.setTarget(map.get(itarget));
                    if (bi instanceof Select) {
                        InstructionHandle[] itargets = ((Select) bi).getTargets();
                        InstructionHandle[] ctargets = ((Select) bc).getTargets();
                        for (int j = 0; j < itargets.length; j++) {
                            ctargets[j] = map.get(itargets[j]);
                        }
                    }
                }
                ih2 = ih2.getNext();
                instructionHandle2 = ch.getNext();
            } else {
                return il;
            }
        }
    }

    public void delete(Instruction i) throws TargetLostException {
        InstructionHandle ih = findInstruction1(i);
        if (ih == null) {
            throw new ClassGenException("Instruction " + i + " is not contained in this list.");
        }
        delete(ih);
    }

    public void delete(Instruction from, Instruction to) throws TargetLostException {
        InstructionHandle fromIh = findInstruction1(from);
        if (fromIh == null) {
            throw new ClassGenException("Instruction " + from + " is not contained in this list.");
        }
        InstructionHandle toIh = findInstruction2(to);
        if (toIh == null) {
            throw new ClassGenException("Instruction " + to + " is not contained in this list.");
        }
        delete(fromIh, toIh);
    }

    public void delete(InstructionHandle ih) throws TargetLostException {
        remove(ih.getPrev(), ih.getNext());
    }

    public void delete(InstructionHandle from, InstructionHandle to) throws TargetLostException {
        remove(from.getPrev(), to.getNext());
    }

    public void dispose() {
        InstructionHandle instructionHandle = this.end;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                ih.dispose();
                instructionHandle = ih.getPrev();
            } else {
                clear();
                return;
            }
        }
    }

    public InstructionHandle findHandle(int pos) {
        int[] positions = this.bytePositions;
        InstructionHandle ih = this.start;
        for (int i = 0; i < this.length; i++) {
            if (positions[i] == pos) {
                return ih;
            }
            ih = ih.getNext();
        }
        return null;
    }

    private InstructionHandle findInstruction1(Instruction i) {
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                if (ih.getInstruction() != i) {
                    instructionHandle = ih.getNext();
                } else {
                    return ih;
                }
            } else {
                return null;
            }
        }
    }

    private InstructionHandle findInstruction2(Instruction i) {
        InstructionHandle instructionHandle = this.end;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                if (ih.getInstruction() != i) {
                    instructionHandle = ih.getPrev();
                } else {
                    return ih;
                }
            } else {
                return null;
            }
        }
    }

    public byte[] getByteCode() {
        setPositions();
        ByteArrayOutputStream b = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream(b);
        try {
            for (InstructionHandle ih = this.start; ih != null; ih = ih.getNext()) {
                Instruction i = ih.getInstruction();
                i.dump(out);
            }
            out.flush();
            return b.toByteArray();
        } catch (IOException e) {
            System.err.println(e);
            return ArrayUtils.EMPTY_BYTE_ARRAY;
        }
    }

    public InstructionHandle getEnd() {
        return this.end;
    }

    public InstructionHandle[] getInstructionHandles() {
        InstructionHandle[] ihs = new InstructionHandle[this.length];
        InstructionHandle ih = this.start;
        for (int i = 0; i < this.length; i++) {
            ihs[i] = ih;
            ih = ih.getNext();
        }
        return ihs;
    }

    public int[] getInstructionPositions() {
        return this.bytePositions;
    }

    public Instruction[] getInstructions() {
        List<Instruction> instructions = new ArrayList<>();
        try {
            ByteSequence bytes = new ByteSequence(getByteCode());
            Throwable th = null;
            while (bytes.available() > 0) {
                try {
                    try {
                        instructions.add(Instruction.readInstruction(bytes));
                    } finally {
                    }
                } finally {
                }
            }
            if (bytes != null) {
                if (0 != 0) {
                    try {
                        bytes.close();
                    } catch (Throwable th2) {
                        th.addSuppressed(th2);
                    }
                } else {
                    bytes.close();
                }
            }
            return (Instruction[]) instructions.toArray(Instruction.EMPTY_ARRAY);
        } catch (IOException e) {
            throw new ClassGenException(e.toString(), e);
        }
    }

    public int getLength() {
        return this.length;
    }

    public InstructionHandle getStart() {
        return this.start;
    }

    public BranchHandle insert(BranchInstruction i) {
        BranchHandle ih = BranchHandle.getBranchHandle(i);
        insert(ih);
        return ih;
    }

    public InstructionHandle insert(CompoundInstruction c) {
        return insert(c.getInstructionList());
    }

    public InstructionHandle insert(Instruction i) {
        InstructionHandle ih = InstructionHandle.getInstructionHandle(i);
        insert(ih);
        return ih;
    }

    public InstructionHandle insert(Instruction i, CompoundInstruction c) {
        return insert(i, c.getInstructionList());
    }

    public InstructionHandle insert(Instruction i, Instruction j) {
        return insert(i, new InstructionList(j));
    }

    public InstructionHandle insert(Instruction i, InstructionList il) {
        InstructionHandle ih = findInstruction1(i);
        if (ih == null) {
            throw new ClassGenException("Instruction " + i + " is not contained in this list.");
        }
        return insert(ih, il);
    }

    private void insert(InstructionHandle ih) {
        if (isEmpty()) {
            this.end = ih;
            this.start = ih;
            ih.setNext(ih.setPrev(null));
        } else {
            this.start.setPrev(ih);
            ih.setNext(this.start);
            ih.setPrev(null);
            this.start = ih;
        }
        this.length++;
    }

    public BranchHandle insert(InstructionHandle ih, BranchInstruction i) {
        BranchHandle bh = BranchHandle.getBranchHandle(i);
        InstructionList il = new InstructionList();
        il.append(bh);
        insert(ih, il);
        return bh;
    }

    public InstructionHandle insert(InstructionHandle ih, CompoundInstruction c) {
        return insert(ih, c.getInstructionList());
    }

    public InstructionHandle insert(InstructionHandle ih, Instruction i) {
        return insert(ih, new InstructionList(i));
    }

    public InstructionHandle insert(InstructionHandle ih, InstructionList il) {
        if (il == null) {
            throw new ClassGenException("Inserting null InstructionList");
        }
        if (il.isEmpty()) {
            return ih;
        }
        InstructionHandle prev = ih.getPrev();
        InstructionHandle ret = il.start;
        ih.setPrev(il.end);
        il.end.setNext(ih);
        il.start.setPrev(prev);
        if (prev != null) {
            prev.setNext(il.start);
        } else {
            this.start = il.start;
        }
        this.length += il.length;
        il.clear();
        return ret;
    }

    public InstructionHandle insert(InstructionList il) {
        if (isEmpty()) {
            append(il);
            return this.start;
        }
        return insert(this.start, il);
    }

    public boolean isEmpty() {
        return this.start == null;
    }

    @Override // java.lang.Iterable
    public Iterator<InstructionHandle> iterator() {
        return new Iterator<InstructionHandle>() { // from class: org.apache.bcel.generic.InstructionList.1
            private InstructionHandle ih;

            {
                this.ih = InstructionList.this.start;
            }

            @Override // java.util.Iterator
            public boolean hasNext() {
                return this.ih != null;
            }

            /* JADX WARN: Can't rename method to resolve collision */
            @Override // java.util.Iterator
            public InstructionHandle next() throws NoSuchElementException {
                if (this.ih == null) {
                    throw new NoSuchElementException();
                }
                InstructionHandle i = this.ih;
                this.ih = this.ih.getNext();
                return i;
            }

            @Override // java.util.Iterator
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    public void move(InstructionHandle ih, InstructionHandle target) {
        move(ih, ih, target);
    }

    public void move(InstructionHandle start, InstructionHandle end, InstructionHandle target) {
        if (start == null || end == null) {
            throw new ClassGenException("Invalid null handle: From " + start + " to " + end);
        }
        if (target == start || target == end) {
            throw new ClassGenException("Invalid range: From " + start + " to " + end + " contains target " + target);
        }
        InstructionHandle instructionHandle = start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != end.getNext()) {
                if (ih == null) {
                    throw new ClassGenException("Invalid range: From " + start + " to " + end);
                }
                if (ih != target) {
                    instructionHandle = ih.getNext();
                } else {
                    throw new ClassGenException("Invalid range: From " + start + " to " + end + " contains target " + target);
                }
            } else {
                InstructionHandle prev = start.getPrev();
                InstructionHandle next = end.getNext();
                if (prev != null) {
                    prev.setNext(next);
                } else {
                    this.start = next;
                }
                if (next != null) {
                    next.setPrev(prev);
                } else {
                    this.end = prev;
                }
                start.setPrev(end.setNext(null));
                if (target == null) {
                    if (this.start != null) {
                        this.start.setPrev(end);
                    }
                    end.setNext(this.start);
                    this.start = start;
                    return;
                }
                InstructionHandle next2 = target.getNext();
                target.setNext(start);
                start.setPrev(target);
                end.setNext(next2);
                if (next2 != null) {
                    next2.setPrev(end);
                    return;
                } else {
                    this.end = end;
                    return;
                }
            }
        }
    }

    public void redirectBranches(InstructionHandle oldTarget, InstructionHandle newTarget) {
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                Instruction i = ih.getInstruction();
                if (i instanceof BranchInstruction) {
                    BranchInstruction b = (BranchInstruction) i;
                    InstructionHandle target = b.getTarget();
                    if (target == oldTarget) {
                        b.setTarget(newTarget);
                    }
                    if (b instanceof Select) {
                        InstructionHandle[] targets = ((Select) b).getTargets();
                        for (int j = 0; j < targets.length; j++) {
                            if (targets[j] == oldTarget) {
                                ((Select) b).setTarget(j, newTarget);
                            }
                        }
                    }
                }
                instructionHandle = ih.getNext();
            } else {
                return;
            }
        }
    }

    public void redirectExceptionHandlers(CodeExceptionGen[] exceptions, InstructionHandle oldTarget, InstructionHandle newTarget) {
        for (CodeExceptionGen exception : exceptions) {
            if (exception.getStartPC() == oldTarget) {
                exception.setStartPC(newTarget);
            }
            if (exception.getEndPC() == oldTarget) {
                exception.setEndPC(newTarget);
            }
            if (exception.getHandlerPC() == oldTarget) {
                exception.setHandlerPC(newTarget);
            }
        }
    }

    public void redirectLocalVariables(LocalVariableGen[] lg, InstructionHandle oldTarget, InstructionHandle newTarget) {
        for (LocalVariableGen element : lg) {
            InstructionHandle start = element.getStart();
            InstructionHandle end = element.getEnd();
            if (start == oldTarget) {
                element.setStart(newTarget);
            }
            if (end == oldTarget) {
                element.setEnd(newTarget);
            }
        }
    }

    private void remove(InstructionHandle prev, InstructionHandle next) throws TargetLostException {
        InstructionHandle first;
        InstructionHandle last;
        if (prev == null && next == null) {
            first = this.start;
            last = this.end;
            this.end = null;
            this.start = null;
        } else {
            if (prev == null) {
                first = this.start;
                this.start = next;
            } else {
                first = prev.getNext();
                prev.setNext(next);
            }
            if (next == null) {
                last = this.end;
                this.end = prev;
            } else {
                last = next.getPrev();
                next.setPrev(prev);
            }
        }
        first.setPrev(null);
        last.setNext(null);
        List<InstructionHandle> targetVec = new ArrayList<>();
        InstructionHandle instructionHandle = first;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih == null) {
                break;
            }
            ih.getInstruction().dispose();
            instructionHandle = ih.getNext();
        }
        StringBuilder buf = new StringBuilder("{ ");
        InstructionHandle instructionHandle2 = first;
        while (true) {
            InstructionHandle ih2 = instructionHandle2;
            if (ih2 == null) {
                break;
            }
            InstructionHandle next2 = ih2.getNext();
            this.length--;
            if (ih2.hasTargeters()) {
                targetVec.add(ih2);
                buf.append(ih2.toString(true)).append(" ");
                ih2.setNext(ih2.setPrev(null));
            } else {
                ih2.dispose();
            }
            instructionHandle2 = next2;
        }
        buf.append("}");
        if (!targetVec.isEmpty()) {
            throw new TargetLostException((InstructionHandle[]) targetVec.toArray(InstructionHandle.EMPTY_ARRAY), buf.toString());
        }
    }

    public void removeObserver(InstructionListObserver o) {
        if (this.observers != null) {
            this.observers.remove(o);
        }
    }

    public void replaceConstantPool(ConstantPoolGen oldCp, ConstantPoolGen newCp) {
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                Instruction i = ih.getInstruction();
                if (i instanceof CPInstruction) {
                    CPInstruction ci = (CPInstruction) i;
                    Constant c = oldCp.getConstant(ci.getIndex());
                    ci.setIndex(newCp.addConstant(c, oldCp));
                }
                instructionHandle = ih.getNext();
            } else {
                return;
            }
        }
    }

    public void setPositions() {
        setPositions(false);
    }

    public void setPositions(boolean check) {
        int maxAdditionalBytes = 0;
        int additionalBytes = 0;
        int index = 0;
        int count = 0;
        int[] pos = new int[this.length];
        if (check) {
            InstructionHandle instructionHandle = this.start;
            while (true) {
                InstructionHandle ih = instructionHandle;
                if (ih == null) {
                    break;
                }
                Instruction i = ih.getInstruction();
                if (i instanceof BranchInstruction) {
                    Instruction inst = ((BranchInstruction) i).getTarget().getInstruction();
                    if (!contains(inst)) {
                        throw new ClassGenException("Branch target of " + Const.getOpcodeName(i.getOpcode()) + ":" + inst + " not in instruction list");
                    }
                    if (i instanceof Select) {
                        InstructionHandle[] targets = ((Select) i).getTargets();
                        for (InstructionHandle target : targets) {
                            inst = target.getInstruction();
                            if (!contains(inst)) {
                                throw new ClassGenException("Branch target of " + Const.getOpcodeName(i.getOpcode()) + ":" + inst + " not in instruction list");
                            }
                        }
                    }
                    if (!(ih instanceof BranchHandle)) {
                        throw new ClassGenException("Branch instruction " + Const.getOpcodeName(i.getOpcode()) + ":" + inst + " not contained in BranchHandle.");
                    }
                }
                instructionHandle = ih.getNext();
            }
        }
        InstructionHandle instructionHandle2 = this.start;
        while (true) {
            InstructionHandle ih2 = instructionHandle2;
            if (ih2 != null) {
                Instruction i2 = ih2.getInstruction();
                ih2.setPosition(index);
                int i3 = count;
                count++;
                pos[i3] = index;
                switch (i2.getOpcode()) {
                    case 167:
                    case 168:
                        maxAdditionalBytes += 2;
                        break;
                    case 170:
                    case 171:
                        maxAdditionalBytes += 3;
                        break;
                }
                index += i2.getLength();
                instructionHandle2 = ih2.getNext();
            } else {
                InstructionHandle instructionHandle3 = this.start;
                while (true) {
                    InstructionHandle ih3 = instructionHandle3;
                    if (ih3 != null) {
                        additionalBytes += ih3.updatePosition(additionalBytes, maxAdditionalBytes);
                        instructionHandle3 = ih3.getNext();
                    } else {
                        int count2 = 0;
                        int index2 = 0;
                        InstructionHandle instructionHandle4 = this.start;
                        while (true) {
                            InstructionHandle ih4 = instructionHandle4;
                            if (ih4 != null) {
                                Instruction i4 = ih4.getInstruction();
                                ih4.setPosition(index2);
                                int i5 = count2;
                                count2++;
                                pos[i5] = index2;
                                index2 += i4.getLength();
                                instructionHandle4 = ih4.getNext();
                            } else {
                                this.bytePositions = new int[count2];
                                System.arraycopy(pos, 0, this.bytePositions, 0, count2);
                                return;
                            }
                        }
                    }
                }
            }
        }
    }

    public int size() {
        return this.length;
    }

    public String toString() {
        return toString(true);
    }

    public String toString(boolean verbose) {
        StringBuilder buf = new StringBuilder();
        InstructionHandle instructionHandle = this.start;
        while (true) {
            InstructionHandle ih = instructionHandle;
            if (ih != null) {
                buf.append(ih.toString(verbose)).append("\n");
                instructionHandle = ih.getNext();
            } else {
                return buf.toString();
            }
        }
    }

    public void update() {
        if (this.observers != null) {
            for (InstructionListObserver observer : this.observers) {
                observer.notify(this);
            }
        }
    }
}
